<?php
/**
 * 2007-2023 PrestaShop and Contributors
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2023 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 * International Registered Trademark & Property of PrestaShop SA
 */
class GlotioLanguagesResourceV1 extends GlotioRestApiResource
{
    public function get(array $params)
    {
        return GlotioPsLanguages::getLanguagesData();
    }

    public function post(array $params)
    {
        if (!is_array($params) || !$params) {
            throw new GlotioException('Expected non empty JSON array in request body', 400);
        }

        $iso_code = isset($params['iso_code']) ? $params['iso_code'] : null;
        if (!Validate::isLanguageIsoCode($iso_code)) {
            throw new GlotioException("Invalid provided iso_code value '$iso_code'", 400);
        }

        if (in_array($iso_code, GlotioPsLanguages::getIsos())) {
            throw new GlotioException('This language is already installed', 400);
        }

        if (GlotioUtils::isPs17OrGreater()) {
            GlotioPsKernel::getInstance();
        }

        try {
            $response = GlotioPsLanguages::addLanguage($iso_code);
            if (!isset($response['success'])) {
                return $this->returnInstallError($response);
            }

            return $response;
        } catch (Error $ex) {
            return $this->returnInstallError($ex);
        } catch (Exception $ex) {
            return $this->returnInstallError($ex);
        }
    }

    public function put(array $params)
    {
        if (!is_array($params) || !$params) {
            throw new GlotioException('Expected non empty JSON array in request body', 400);
        }

        $iso_code = Tools::getValue('iso_code', null);
        if (!Validate::isLanguageIsoCode($iso_code)) {
            throw new GlotioException("Invalid provided iso_code value '$iso_code'", 400);
        }

        if (!in_array($iso_code, GlotioPsLanguages::getIsos())) {
            throw new GlotioException('This language is not installed yet', 400);
        }

        $active = isset($params['active']) ? $params['active'] : null;
        $default = isset($params['default']) ? $params['default'] : null;

        if ($active === null && $default === null) {
            throw new GlotioException("Param 'active' or 'default' must be provided", 400);
        }

        if ($active !== null && $default !== null) {
            throw new GlotioException('Set default and activate/deactivate language at once is not permitted', 400);
        }

        $id_lang = GlotioPsLanguages::isoToIdLang($iso_code);

        $language = new Language($id_lang);
        $default_id_lang = (int) Configuration::get('PS_LANG_DEFAULT');

        if ($active !== null) {
            return $this->toggleLanguageStatus($language, $default_id_lang, $active);
        }

        return $this->setDefaultLanguage($language, $default, $default_id_lang);
    }

    /**
     * @param Language $language
     * @param int $default_id_lang
     * @param bool $active
     *
     * @return array
     *
     * @throws GlotioException
     */
    private function toggleLanguageStatus($language, $default_id_lang, $active)
    {
        if ($language->id === $default_id_lang) {
            throw new GlotioException('Can not change the state of the default language', 400);
        }

        if ($active && $language->active) {
            return array(
                'success' => true,
                'message' => 'This language is already activated',
            );
        }

        if (!$active && !$language->active) {
            return array(
                'success' => true,
                'message' => 'This language is already deactivated',
            );
        }

        try {
            return GlotioPsLanguages::toggleLanguageStatus($language, $active);
        } catch (Error $ex) {
            return $this->returnUpdateError($ex);
        } catch (Exception $ex) {
            return $this->returnUpdateError($ex);
        }
    }

    /**
     * @param Language $language
     * @param bool $default
     * @param int $default_id_lang
     *
     * @return array
     */
    private function setDefaultLanguage($language, $default, $default_id_lang)
    {
        if (!$default) {
            throw new GlotioException("Param 'default' must be true", 400);
        }

        if ($language->id === $default_id_lang) {
            return array(
                'success' => true,
                'message' => sprintf("The language with iso '%s' is already the default language", $language->iso_code),
            );
        }

        try {
            return GlotioPsLanguages::setDefaultLanguage($language);
        } catch (Error $ex) {
            return $this->returnUpdateError($ex);
        } catch (Exception $ex) {
            return $this->returnUpdateError($ex);
        }
    }

    /**
     * @param array|Exception|Error $error
     *
     * @return array
     */
    private function returnInstallError($error)
    {
        return array(
            'success' => false,
            'message' => 'Error while installing language pack',
            'errors' => is_array($error) ? $error : $error->getMessage(),
        );
    }

    /**
     * @param Exception|Error $error
     *
     * @return array
     */
    private function returnUpdateError($error)
    {
        return array(
            'success' => false,
            'message' => 'Error while updating language configuration',
            'error' => $error->getMessage(),
        );
    }
}
