<?php
/**
 * 2007-2023 PrestaShop and Contributors
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2023 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 * International Registered Trademark & Property of PrestaShop SA
 */
abstract class GlotioPs17TranslationsHelper
{
    /**
     * Process submitted records to TranslationSource::addRecords for Prestashop 1.7 translation files.
     *
     * @param array $records
     * @param array $originals
     * @param string|null $theme
     *
     * @return array
     *
     * @throws GlotioException
     */
    public static function addRecordsAsTranslations(array $records, array $originals, $theme = null)
    {
        // Group texts by _id and language
        $translationTextsMap = array();

        foreach ($records as $record) {
            if (!isset($record['_id'], $record['texts'])) {
                throw new GlotioException('Incorrect JSON format provided, should have [_id] and [texts] properties');
            }

            $_id = $record['_id'];
            $texts = $record['texts'];

            foreach ($texts as $iso_lang => $translated_fields) {
                foreach ($translated_fields as $field => $value) {
                    // Parses strings like: <ModulesBannerShop>92fbf0e5d97b8afd7e73126b52bdc4bb
                    preg_match('/<(.+)>(.+)/i', $field, $matches);
                    if (count($matches) < 3) {
                        continue;
                    }

                    $domain = $matches[1];

                    $translationMessage = array(
                        'field' => $field,
                        'original' => isset($originals[$field]) ? $originals[$field] : null,
                        'translation' => $value,
                        'domain' => $domain,
                        'theme' => $theme,
                    );
                    $translationTextsMap[$_id][$iso_lang][$field] = $translationMessage;
                }
            }
        }

        // Try to save texts to translation table
        $results = array();
        $translatedIsos = array();

        foreach ($translationTextsMap as $_id => $translationTextsByIso) {
            foreach ($translationTextsByIso as $iso_lang => $translationMessages) {
                $translatedIsos[$iso_lang] = $iso_lang;
                $id_lang = GlotioPsLanguages::isoToIdLang($iso_lang);

                $errors = array();
                foreach ($translationMessages as $translationMessage) {
                    $domain = $translationMessage['domain'];
                    $key = htmlspecialchars_decode($translationMessage['original'], ENT_QUOTES);
                    $translation = $translationMessage['translation'];
                    $theme = $translationMessage['theme'];

                    if (!$key) {
                        $errors[] = 'Cannot found original text. ' . json_encode($translationMessage);

                        continue;
                    }

                    if (GlotioUtils::is1780OrGreatVersion() && $theme === 'classic') {
                        $themeCondition = '`theme` IS NULL';
                        $theme = null;
                    } else {
                        $themeCondition = '`theme` ' . (($theme) ? "= '" . pSQL($theme) . "'" : 'IS NULL');
                    }

                    // Get existing ID from translation table
                    $query = new DbQuery();
                    $query->select('`id_translation`')
                        ->from('translation')
                        ->where('`id_lang` = ' . (int) $id_lang)
                        ->where("`domain` = '" . pSQL($domain) . "'")
                        ->where("`key` = '" . pSQL($key) . "'")
                        ->where($themeCondition)
                        ->orderBy('`id_translation` DESC');
                    $current_id = (int) Db::getInstance()->getValue($query);

                    // Insert or update existing value in translation table
                    $values = array(
                        'id_translation' => $current_id ? $current_id : null,
                        'id_lang' => $id_lang,
                        'key' => $key,
                        'translation' => $translation,
                        'domain' => $domain,
                        'theme' => $theme,
                    );

                    $query = new GlotioInsertQueryBuilder();
                    $query->setTable(_DB_PREFIX_ . 'translation')
                        ->setColumns(array('id_translation', 'id_lang', 'key', 'translation', 'domain', 'theme'))
                        ->setValues(array($values))
                        ->setUpdatefields(array('id_lang', 'key', 'translation', 'domain', 'theme'));

                    $query_result = Db::getInstance()->execute($query->build());
                    if (!$query_result) {
                        $msg = 'Error while saving translation. ' . Db::getInstance()->getMsgError();
                        $errors[md5($msg)] = $msg;
                    }
                }

                // Info to generate result record
                $fields = array($iso_lang => array_column($translationMessages, 'field'));

                if ($errors) {
                    $result = GlotioAbstractTranslationSource::createTranslationUploadResultError($_id, $fields, implode('|', array_values($errors)));
                } else {
                    $result = GlotioAbstractTranslationSource::createTranslationUploadResultSuccess($_id, $fields);
                }

                $results[] = $result;
            }
        }

        if ($results) {
            if ($theme === GlotioUtils::parentThemeName() && GlotioUtils::is1780OrGreatVersion() && $translatedIsos) {
                $childTheme = GlotioUtils::themeName();

                foreach ($translatedIsos as $iso) {
                    $locale = GlotioPsLanguages::isoToLocale($iso);

                    $exporter = GlotioPsKernel::themeExporter();
                    $zipFile = $exporter->createZipArchive($theme, $locale, _PS_ROOT_DIR_ . DIRECTORY_SEPARATOR);

                    $extractPath = _PS_ROOT_DIR_ . DIRECTORY_SEPARATOR . 'themes' . DIRECTORY_SEPARATOR . $childTheme . DIRECTORY_SEPARATOR . 'translations' . DIRECTORY_SEPARATOR . $locale;
                    $fileSystem = new \Symfony\Component\Filesystem\Filesystem();
                    $fileSystem->mkdir($extractPath);

                    $zip = new ZipArchive;
                    if ($zip->open($zipFile) !== true) {
                        throw new GlotioException("Error while trying to extract $theme translations from $zipFile");
                    }

                    $zip->extractTo($extractPath);
                    $zip->close();
                }
            }

            $cacheRefresh = GlotioPsKernel::prestashopCacheRefresh();
            if ($cacheRefresh) {
                try {
                    $cacheRefresh->addCacheClear();
                    $cacheRefresh->execute();
                } catch (Exception $exception) {
                    // Ignore exception, should be a permissions issue
                }
            }
        }

        return $results;
    }

    /**
     * Retrieves theme translation texts in PrestaShop
     *
     * @param string $theme_name
     *
     * @return array Format array(texts => array(), originals => array())
     *
     * @throws GlotioException
     */
    public static function getThemeTranslations($theme_name)
    {
        if (GlotioUtils::is1780OrGreatVersion()) {
            return self::getThemeTranslationsPrestashop1780OrHigher($theme_name);
        } else {
            return self::getThemeTranslationsPrestashop1700To1778($theme_name);
        }
    }

    /**
     * Retrieves theme translation texts in PrestaShop (>= 1.7.0.0 && <= 1.7.7.8 Versions)
     *
     * @param string $theme_name
     *
     * @return array{texts: array, originals: array}
     *
     * @throws GlotioException
     */
    private static function getThemeTranslationsPrestashop1700To1778($theme_name)
    {
        $cache = self::textsCache();
        $translationService = GlotioPsKernel::translationService();

        $texts = array();
        $originals = array();

        foreach (GlotioPsLanguages::getIsos() as $iso) {
            $cacheKey = implode('|', array('theme', $theme_name, $iso));
            $translationsData = $cache->get($cacheKey);
            if (!$translationsData) {
                try {
                    $catalogue = $translationService->getTranslationsCatalogue($iso, 'themes', $theme_name);
                } catch (Exception $ex) {
                    continue;
                }

                $translationsData = self::extractCatalogueArrayTranslations($catalogue, true);

                $cache->add($cacheKey, $translationsData);
            }

            if (!$translationsData) {
                continue;
            }

            if (!isset($texts[$iso])) {
                $texts[$iso] = array();
            }

            $texts[$iso] = array_merge($texts[$iso], $translationsData['translations']);
            $originals = array_merge($originals, $translationsData['defaults']);
        }

        return compact('texts', 'originals');
    }

    /**
     * Retrieves theme translation texts in PrestaShop (>= 1.7.8.0 Versions).
     *
     * @param string $theme_name
     *
     * @return array{texts: array, originals: array}
     *
     * @throws GlotioException
     */
    private static function getThemeTranslationsPrestashop1780OrHigher($theme_name)
    {
        $cache = self::textsCache();
        $translationsCatalogue = GlotioPsKernel::translationsCatalogue();
        GlotioPsKernel::catalogueProviderFactory();

        if ($theme_name === 'classic') {
            // Prestashop translator show domains from FrontofficeProvider but load translations from CoreProvider
            // So it can't detect new texts in template files
            // Loading domains and texts from FrontofficeProvider is faster and produces the same output
            // If this fails in the future check TranslationController to for code changes
            $providerDefinition = new \PrestaShop\PrestaShop\Core\Translation\Storage\Provider\Definition\FrontofficeProviderDefinition();
        } else {
            // ThemeProvider can detect new texts in template files
            $providerDefinition = new \PrestaShop\PrestaShop\Core\Translation\Storage\Provider\Definition\ThemeProviderDefinition($theme_name);
        }

        $texts = array();
        $originals = array();

        foreach (GlotioPsLanguages::getIsos() as $iso) {
            $locale = GlotioPsLanguages::isoToLocale($iso);

            $cacheKey = implode('|', array('theme', $theme_name, $iso));
            $translationsData = $cache->get($cacheKey);
            if (!$translationsData) {
                try {
                    $catalogue = $translationsCatalogue->getRawCatalogue($providerDefinition, $locale, array());
                } catch (Exception $ex) {
                    continue;
                }

                $translationsData = self::extractCatalogueTranslations($catalogue);

                $cache->add($cacheKey, $translationsData);
            }

            if (!isset($texts[$iso])) {
                $texts[$iso] = array();
            }

            $texts[$iso] = array_merge($texts[$iso], $translationsData['translations']);
            $originals = array_merge($originals, $translationsData['defaults']);
        }

        return compact('texts', 'originals');
    }

    /**
     * Retrieves module translation texts using the new translation system for Prestashop 1.7.
     *
     * @param string $module_name
     *
     * @return array{texts: array, originals: array}
     *
     * @throws GlotioException
     */
    public static function getModuleTranslations($module_name)
    {
        if (GlotioUtils::is1780OrGreatVersion()) {
            return self::getModuleTranslationsPrestashop1780OrHigher($module_name);
        } else {
            return self::getModuleTranslationsPrestashop1770To1778($module_name);
        }
    }

    /**
     * Retrieves module translation texts in PrestaShop (>= 1.7.0.0 && <= 1.7.7.8 Versions)
     *
     * @param string $module_name
     *
     * @return array{texts: array, originals: array}
     *
     * @throws GlotioException
     */
    public static function getModuleTranslationsPrestashop1770To1778($module_name)
    {
        $moduleProvider = GlotioPsKernel::moduleProvider();
        $moduleProvider->setModuleName($module_name);

        $texts = array();
        $originals = array();

        foreach (GlotioPsLanguages::getIsos() as $iso) {
            $locale = GlotioPsLanguages::isoToLocale($iso);

            try {
                $treeBuilder = new \PrestaShopBundle\Translation\View\TreeBuilder($locale, null);
                $catalogue = $treeBuilder->makeTranslationArray($moduleProvider);
            } catch (Exception $ex) {
                continue;
            }

            $translationsData = self::extractCatalogueArrayTranslations($catalogue, true);

            if (!isset($texts[$iso])) {
                $texts[$iso] = array();
            }

            $texts[$iso] = array_merge($texts[$iso], $translationsData['translations']);
            $originals = array_merge($originals, $translationsData['defaults']);
        }

        return compact('texts', 'originals');
    }

    /**
     * Retrieves module translation texts in PrestaShop (<= 1.7.8.0 Versions)
     *
     * @param string $module_name
     *
     * @return array{texts: array, originals: array}
     *
     * @throws GlotioException
     */
    public static function getModuleTranslationsPrestashop1780OrHigher($module_name)
    {
        $translationCatalogue = GlotioPsKernel::translationsCatalogue();

        $providerDefinition = new \PrestaShop\PrestaShop\Core\Translation\Storage\Provider\Definition\ModuleProviderDefinition($module_name);

        $texts = array();
        $originals = array();

        foreach (GlotioPsLanguages::getIsos() as $iso) {
            $locale = GlotioPsLanguages::isoToLocale($iso);

            try {
                $catalogue = $translationCatalogue->getRawCatalogue($providerDefinition, $locale, array());
            } catch (Exception $ex) {
                continue;
            }

            $translationsData = self::extractCatalogueTranslations($catalogue);
            if (!$translationsData) {
                continue;
            }

            if (!isset($texts[$iso])) {
                $texts[$iso] = array();
            }

            $texts[$iso] = array_merge($texts[$iso], $translationsData['translations']);
            $originals = array_merge($originals, $translationsData['defaults']);
        }

        return compact('texts', 'originals');
    }

    /**
     * Extract translation data (>= 1.7.0.0 && <= 1.7.7.8 Versions)
     *
     * @param array $domainTranslations
     * @param bool $normalizeDomains
     *
     * @return array{translations: array, defaults: array}|null
     */
    private static function extractCatalogueArrayTranslations(array $catalogue, $normalizeDomains)
    {
        // To return texts in the same order that Prestashop translator backoffice view
        $domains = array_keys($catalogue);
        sort($domains);

        $translations = array();
        $defaults = array();
        foreach ($domains as $domainName) {
            $domainCatalogue = $catalogue[$domainName];
            if ($normalizeDomains) {
                $domainName = self::normalizeDomain($domainName);
            }

            foreach ($domainCatalogue as $original => $messages) {
                if ($original === '__metadata') {
                    continue;
                }

                $translation = ($messages['db']) ? $messages['db'] : $messages['xlf'];
                $key = self::buildTextKey($domainName, $original);
                $translations[$key] = $translation;
                $defaults[$key] = $original;
            }
        }

        return compact('translations', 'defaults');
    }

    /**
     * Extract translation texts (<= 1.7.8.0 Versions) from Catalogue.
     *
     * @param \PrestaShop\PrestaShop\Core\Translation\Builder\Map\Catalogue $catalogue
     *
     * @return array{translations: array<string, string>, defaults: array<string, string>}
     */
    private static function extractCatalogueTranslations($catalogue)
    {
        // To return texts in the same order that Prestashop translator backoffice view
        $domains = array();
        foreach ($catalogue->getDomains() as $domain) {
            $domains[] = $domain->getDomainName();
        }
        sort($domains);

        $translations = array();
        $defaults = array();
        foreach ($domains as $domainName) {
            $domain = $catalogue->getDomain($domainName);
            foreach ($domain->getMessages() as $message) {
                $original = $message->getKey();
                $translation = $message->getTranslation();
                $key = self::buildTextKey($domainName, $original);
                $translations[$key] = $translation;
                $defaults[$key] = $original;
            }
        }

        return compact('translations', 'defaults');
    }

    /**
     * @param string $domain
     * @param string $text
     *
     * @return string
     */
    private static function buildTextKey($domain, $text)
    {
        return "<$domain>" . md5($text);
    }

    /**
     * Fix domains ending with locale like .es-ES in oldest 1.7 versions.
     *
     * @param string $domain
     *
     * @return string
     */
    private static function normalizeDomain($domain)
    {
        $parts = explode('.', $domain);

        return reset($parts);
    }

    private static function textsCache()
    {
        static $cache;

        if (is_null($cache)) {
            $cache = GlotioTranslationCache::createForThemeTexts();
        }

        return $cache;
    }
}
