<?php
/**
 * 2007-2023 PrestaShop and Contributors
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2023 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 * International Registered Trademark & Property of PrestaShop SA
 */
/**
 * Translation util methods.
 */
abstract class GlotioUtils
{
    /**
     * @param int $id_shop
     * @param int $ssl
     * @param bool $relative_protocol
     *
     * @return string
     *
     * @see Link::getBaseLink()
     */
    public static function getBaseLink($id_shop = null, $ssl = null, $relative_protocol = false)
    {
        static $force_ssl = null;

        if ($ssl === null) {
            if ($force_ssl === null) {
                $force_ssl = (Configuration::get('PS_SSL_ENABLED') && Configuration::get('PS_SSL_ENABLED_EVERYWHERE'));
            }
            $ssl = $force_ssl;
        }

        if (Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE') && $id_shop !== null) {
            $shop = new Shop($id_shop);
        } else {
            $shop = Context::getContext()->shop;
        }

        $ssl_enable = Configuration::get('PS_SSL_ENABLED');
        if ($relative_protocol) {
            $base = '//' . ($ssl && $ssl_enable ? $shop->domain_ssl : $shop->domain);
        } else {
            $base = (($ssl && $ssl_enable) ? 'https://' . $shop->domain_ssl : 'http://' . $shop->domain);
        }

        return $base . $shop->getBaseURI();
    }

    /**
     * Extrae una lista de claves de un array.
     *
     * @param array $array
     * @param array $keys
     *
     * @return array
     */
    public static function extractKeys(array $array, array $keys)
    {
        return array_intersect_key($array, array_flip($keys));
    }

    /**
     * Indica si el Prestashop es de la versión 8.0 o no.
     *
     * @return bool
     */
    public static function isPs8()
    {
        return version_compare(_PS_VERSION_, '8.0', '>') && version_compare(_PS_VERSION_, '9.0', '<');
    }

    /**
     * Indica si el Prestashop 1.7 o superior.
     *
     * @return bool
     */
    public static function isPs17OrGreater()
    {
        return version_compare(_PS_VERSION_, '1.7', '>');
    }

    /**
     * @return bool
     */
    public static function is1780OrGreatVersion()
    {
        return version_compare(_PS_VERSION_, '1.7.8', '>=');
    }

    /**
     * Indica si el Prestashop es de la versión 1.7 o no.
     *
     * @return bool
     */
    public static function isPs17()
    {
        return version_compare(_PS_VERSION_, '1.7', '>') && version_compare(_PS_VERSION_, '1.8', '<');
    }

    /**
     * Indica si el Prestashop es de la versión 1.6 o no.
     *
     * @return bool
     */
    public static function isPs16()
    {
        return version_compare(_PS_VERSION_, '1.6', '>') && version_compare(_PS_VERSION_, '1.7', '<');
    }

    /**
     * @return bool
     */
    public static function isPhp53OrGreater()
    {
        $currentPhpVersion = phpversion();
        return version_compare($currentPhpVersion, '5.3', '>=');
    }

    /**
     * Borrar del path pasado la ruta a Prestashop.
     *
     * @param string $filePath
     *
     * @return string
     */
    public static function removePsFromPath($filePath)
    {
        return str_replace(_PS_ROOT_DIR_ . '/', '', $filePath);
    }

    /**
     * Generates a valid random api key.
     *
     * @return string
     */
    public static function getRandomApiKey()
    {
        $chars = '1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $length = 32;

        $new_key_value = '';
        for ($i = 0; $i < $length; $i++) {
            $pos = mt_rand(0, Tools::strlen($chars) - 1);
            $new_key_value .= $chars[$pos];
        }

        return $new_key_value;
    }

    /**
     * Devuelve la primera ruta existente de la lista pasada.
     *
     * @param string[] $paths
     *
     * @return string|null
     */
    public static function getExistingPathFromList(array $paths)
    {
        foreach ($paths as $path) {
            if (file_exists($path)) {
                return $path;
            }
        }

        return null;
    }

    /**
     * Create one level parent directory of passed path if it doesn't exist yet.
     *
     * @param string $filePath
     *
     * @return bool
     *
     * @throws GlotioException
     */
    public static function createParentDirOfPathIfNotExists($filePath)
    {
        $dir = pathinfo($filePath, PATHINFO_DIRNAME);
        if (!is_dir($dir)) {
            $parent_dir = pathinfo($dir, PATHINFO_DIRNAME);
            if (is_dir($parent_dir) && is_writable($parent_dir)) {
                return mkdir($dir, 0777);
            } else {
                throw new GlotioException('Cannot create directory ' . GlotioUtils::removePsFromPath($dir) . ', path is not writable');
            }
        }
    }

    /**
     * @param int|null $shopId
     *
     * @return bool
     */
    public static function isLinkRewriteEnabled($shopId = null)
    {
        return  self::getConfigInt('PS_REWRITING_SETTINGS', $shopId) === 1;
    }

    /**
     * @param int|null $shopId
     *
     * @return bool
     */
    public static function isSslEnabled($shopId = null)
    {
        return self::getConfigInt('PS_SSL_ENABLED', $shopId) === 1;
    }

    /**
     * @return string
     */
    public static function themeName()
    {
        return (string) _THEME_NAME_;
    }

    /**
     * @return string
     */
    public static function parentThemeName()
    {
        $parentThemeName = '';

        if (defined('_PARENT_THEME_NAME_')) {
            $parentThemeName = (string) _PARENT_THEME_NAME_;
        }

        return $parentThemeName;
    }

    /**
     * Define admin directory to prevent an exception when install a "reading from right to left" language.
     * Eg Arabic
     */
    public static function defineAdminDir()
    {
        if (GlotioUtils::isPs16() || defined('_PS_ADMIN_DIR_')) {
            return;
        }

        $prestashopAdminDirectoryPath = null;
        $PrestashopRootDirectoryFiles = scandir(_PS_ROOT_DIR_);

        foreach ($PrestashopRootDirectoryFiles as $file) {
            $fullPath = _PS_ROOT_DIR_ . '/' . $file;
            if (!is_dir($fullPath)) {
                continue;
            }

            $fullIndexFilePath = $fullPath . '/index.php';
            if (!file_exists($fullIndexFilePath)) {
                continue;
            }

            $indexFileContent = Tools::file_get_contents($fullIndexFilePath);
            if (strpos($indexFileContent, "define('_PS_ADMIN_DIR_', __DIR__)")) {
                $prestashopAdminDirectoryPath = $fullPath;

                break;
            }
        }

        if ($prestashopAdminDirectoryPath) {
            define('_PS_ADMIN_DIR_', $prestashopAdminDirectoryPath);
        }
    }

    /**
     * @param string $key
     * @param int|null $shopId
     *
     * @return bool
     */
    private static function getConfigInt($key, $shopId = null)
    {
        return (int) Configuration::get($key, null, null, $shopId);
    }
}
